package model

import (
	pbVars "live-service/app/pb/vars"
)

var rankAndScoreMap map[pbVars.RankType]ScoreType
var scoreAndRankMap map[ScoreType]pbVars.RankType

func ScoreTypeByRankType(rankType pbVars.RankType) ScoreType {
	if scoreType, ok := rankAndScoreMap[rankType]; ok {
		return scoreType
	}
	return ScoreTypeDamage
}

func RankTypeByScoreType(scoreType ScoreType) pbVars.RankType {
	if rankType, ok := scoreAndRankMap[scoreType]; ok {
		return rankType
	}
	return pbVars.RankType_Damage
}

func AllScoreType() []ScoreType {
	result := make([]ScoreType, 0, len(rankAndScoreMap))
	for _, scoreType := range rankAndScoreMap {
		result = append(result, scoreType)
	}
	return result
}

func AllRankType() []pbVars.RankType {
	result := make([]pbVars.RankType, 0, len(rankAndScoreMap))
	for rankType, _ := range rankAndScoreMap {
		result = append(result, rankType)
	}
	return result
}

func addRankScore(rankType pbVars.RankType, scoreType ScoreType) {
	rankAndScoreMap[rankType] = scoreType
	scoreAndRankMap[scoreType] = rankType
}

func init() {
	rankAndScoreMap = make(map[pbVars.RankType]ScoreType)
	scoreAndRankMap = make(map[ScoreType]pbVars.RankType)

	addRankScore(pbVars.RankType_Damage, ScoreTypeDamage)
	addRankScore(pbVars.RankType_DeDamage, ScoreTypeDeDamage)
	addRankScore(pbVars.RankType_General, ScoreTypeGeneral)
	addRankScore(pbVars.RankType_DeGeneral, ScoreTypeDeGeneral)
	addRankScore(pbVars.RankType_KillUnit, ScoreTypeKillUnit)
	addRankScore(pbVars.RankType_DeKillUnit, ScoreTypeDeKillUnit)
	addRankScore(pbVars.RankType_KillPlayer, ScoreTypeKillPlayer)
	addRankScore(pbVars.RankType_DeKillPlayer, ScoreTypeDeKillPlayer)
	addRankScore(pbVars.RankType_Win, ScoreTypeWin)
	addRankScore(pbVars.RankType_Lost, ScoreTypeLost)
	addRankScore(pbVars.RankType_FirstBlood, ScoreTypeFirstBlood)
	addRankScore(pbVars.RankType_DeFirstBlood, ScoreTypeDeFirstBlood)
}
