package coin

import (
	"context"
	"github.com/pkg/errors"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type TransferUserCoinLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewTransferUserCoinLogic(ctx context.Context, svcCtx *svc.ServiceContext) *TransferUserCoinLogic {
	return &TransferUserCoinLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// transferUserCoin 转移用户弹币
func (l *TransferUserCoinLogic) TransferUserCoin(in *pb.TransferUserCoinReq) (*pb.TransferUserCoinResp, error) {
	transResp, err := l.svcCtx.CoinManager.TransferCoin(l.ctx, in)
	if err != nil {
		return nil, errors.Wrap(nerr.NewWithErr(err), "转移弹币失败")
	}
	// 获取 username
	if tmp, err := l.svcCtx.UserPlatformModel.FindDisplayOneByUserId(l.ctx, transResp.UserId); err == nil {
		transResp.Uname = tmp.PUname
		transResp.Avatar = tmp.PAvatar
	}
	if tmp, err := l.svcCtx.UserPlatformModel.FindDisplayOneByUserId(l.ctx, transResp.TargetUserId); err == nil {
		transResp.TargetUname = tmp.PUname
		transResp.TargetAvatar = tmp.PAvatar
	}
	return transResp, nil
}
