package rank

import (
	"context"
	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"
	"live-service/common/nerr"

	"github.com/zeromicro/go-zero/core/logx"
)

type RankPvpLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewRankPvpLogic(ctx context.Context, svcCtx *svc.ServiceContext) *RankPvpLogic {
	return &RankPvpLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

//  rank
func (l *RankPvpLogic) RankPvp(in *pb.RankPvpReq) (*pb.RankPvpResp, error) {
	if !l.svcCtx.GameConfig.Rank.Enabled {
		return nil, nerr.NewError(nerr.RankNotEnabled, "排行榜功能未开启")
	}
	resp := &pb.RankPvpResp{}
	rankRange := Service.RangeRankByType(in.Type, in.TopN)

	resp.Type = rankRange.Type
	resp.Items = make([]*pb.RankPvpResp_Item, 0, len(rankRange.Items))
	for _, item := range rankRange.Items {
		resp.Items = append(resp.Items, &pb.RankPvpResp_Item{
			Uid:    item.UserId,
			Uname:  item.Username,
			Score:  item.Score,
			Avatar: item.Avatar,
		})
	}
	return resp, nil
}
