package user

import (
	"context"
	"github.com/pkg/errors"
	"live-service/common/nerr"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetUserIdByPUidLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGetUserIdByPUidLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetUserIdByPUidLogic {
	return &GetUserIdByPUidLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

// GetUserIdByPUid 通过平台用户id获取系统用户ID
func (l *GetUserIdByPUidLogic) GetUserIdByPUid(in *pb.PlatformUserReq) (*pb.UserIdResp, error) {
	sysUserId, err := l.svcCtx.UserPlatformModel.FindUserIdByPlatform(l.ctx, in.Platform, in.PUid)
	if err != nil {
		return nil, errors.Wrapf(nerr.NewWithCode(nerr.DBError), "获取系统用户ID失败, err:%+v", err)
	}

	return &pb.UserIdResp{
		UserId: sysUserId,
	}, nil
}
