package zhg

import (
	"context"
	"git.noahlan.cn/northlan/ntools-go/gorm-zero/gormx"
	"github.com/pkg/errors"
	"live-service/app/user_center/model"
	"live-service/common/nerr"
	"live-service/common/timex"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type ChangeTitleLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewChangeTitleLogic(ctx context.Context, svcCtx *svc.ServiceContext) *ChangeTitleLogic {
	return &ChangeTitleLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *ChangeTitleLogic) ChangeTitle(in *pb.TitleReq) (*pb.ChangeTitleResp, error) {
	resp := &pb.ChangeTitleResp{
		UserId: in.UserId,
	}

	dbModel, err := l.svcCtx.UserTitleModel.FindOneByUserIdSort(l.ctx, nil, in.UserId, int64(in.Sort))
	if err != nil {
		if errors.Is(err, gormx.ErrNotFound) {
			return nil, errors.Wrap(nerr.NewError(nerr.UserTitleNotFoundErr, "用户没有此称号"), "用户没有此称号")
		}
		return nil, errors.Wrap(nerr.NewWithCode(nerr.DBError), "数据库错误，请联系UP主")
	}
	// 判断过期
	if timex.DayExpire(time.Now(), dbModel.EndTime, bool(dbModel.Forever)) {
		return nil, errors.Wrap(nerr.NewError(nerr.UserTitleExpiresErr, "待切换的称号已过期"), "待切换的称号已过期")
	}

	// 更新当前佩戴称号
	err = l.svcCtx.ZhgUserDetailsModel.Upsert(l.ctx, nil, &model.UpsertParam{
		UserId: in.UserId,
		Title:  &dbModel.TitleId,
	})
	if err != nil {
		if !errors.Is(err, gormx.ErrRowsAffectedZero) {
			return nil, errors.Wrapf(nerr.NewError(nerr.UserChangeTitleErr, "切换称号失败"), "切换称号失败 err:%v", err)
		}
	}

	resp.TitleId = dbModel.TitleId

	var titleName string
	if titleCfg, err := l.svcCtx.GameConfig.Title.FindTitleCfg(dbModel.TitleId); err == nil {
		titleName = titleCfg.Name
	}
	resp.Name = titleName
	return resp, nil
}
