package zhg

import (
	"context"
	"live-service/common/nerr"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type GiveEliteLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGiveEliteLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GiveEliteLogic {
	return &GiveEliteLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GiveEliteLogic) GiveElite(in *pb.GiveEliteReq) (*pb.BuyEliteResp, error) {
	elite, err := l.svcCtx.GameConfig.Elite.FindEliteCfg(in.EliteId)
	if err != nil {
		return nil, nerr.NewError(nerr.EliteConfigNotFoundErr, err.Error())
	}

	duration := in.Duration
	if in.Forever {
		duration = -1
	}
	resp := &pb.BuyEliteResp{
		UserId:   in.UserId,
		EliteId:  elite.Id,
		Cost:     0,
		Duration: duration,
	}
	if duration < 0 {
		duration = -duration
	} else if duration == 0 {
		duration = 1
	}
	err = l.svcCtx.UserEliteModel.Addon(l.ctx, nil, in.UserId, elite.Id, time.Duration(duration)*24*time.Hour, in.Forever)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
