package zhg

import (
	"context"
	"live-service/common/nerr"
	"time"

	"live-service/app/user_center/rpc/internal/svc"
	"live-service/app/user_center/rpc/pb"

	"github.com/zeromicro/go-zero/core/logx"
)

type GiveTitleLogic struct {
	ctx    context.Context
	svcCtx *svc.ServiceContext
	logx.Logger
}

func NewGiveTitleLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GiveTitleLogic {
	return &GiveTitleLogic{
		ctx:    ctx,
		svcCtx: svcCtx,
		Logger: logx.WithContext(ctx),
	}
}

func (l *GiveTitleLogic) GiveTitle(in *pb.GiveTitleReq) (*pb.BuyTitleResp, error) {
	cfg, err := l.svcCtx.GameConfig.Title.FindTitleCfg(in.TitleId)
	if err != nil {
		return nil, nerr.NewError(nerr.EliteConfigNotFoundErr, err.Error())
	}

	duration := in.Duration
	if in.Forever {
		duration = -1
	}
	resp := &pb.BuyTitleResp{
		UserId:   in.UserId,
		TitleId:  cfg.Id,
		Name:     cfg.Name,
		Cost:     0,
		Duration: duration,
	}
	if duration < 0 {
		duration = -duration
	} else if duration == 0 {
		duration = 1
	}
	err = l.svcCtx.UserTitleModel.Addon(l.ctx, nil, in.UserId, cfg.Id, cfg.Type, time.Duration(duration)*24*time.Hour, in.Forever)
	if err != nil {
		return nil, err
	}

	return resp, nil
}
